/*
 * Copyright (c) 2003-2007 Erez Zadok
 * Copyright (c) 2003-2007 Stony Brook University
 * Copyright (c) 2003-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2007 Gopala Surya
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */

#ifndef __NAMECACHE_H
#define __NAMECACHE_H
#include "fist.h"

#ifndef LINUX_KERNEL_LESS_THAN_2_6
#define kdev_t dev_t
#endif

struct namecache {
    spinlock_t nc_lock;
    struct list_head nc_list[0];
};

struct fid {
        struct list_head f_list;
        kdev_t f_dev;
        ino_t f_ino;
};

struct name {
    atomic_t n_refcount;                    // How many trace conds are sharing this object?
    spinlock_t n_lock;                      // locking while traversing
    struct list_head n_list;                // The list of names that share a bucket.
    // (e.g., Mapping .c to this object,
    // using the n_pat as a hash
    // key)
    struct qstr n_pat;                      // Pattern or pathname to match
    int n_buckets;                          // Number of buckets to use for hashing
    struct list_head n_fid[0];              // OOB Buckets
};

struct namecache* namecache_init(void);
void namecache_release(struct namecache *namecache);
struct name *add_pattern(struct namecache *namecache, struct qstr *pattern, int ext);
void release_pattern(struct namecache *namecache, struct name *pattern);

int add_inode(struct namecache *namecache, struct dentry *dentry, struct inode *inode);
void delete_inode(struct namecache *namecache, struct dentry *dentry, struct inode *inode);
void delete_inode_global(struct namecache *namecache, struct inode *inode);
void namecache_rename(struct namecache *namecache, struct dentry *old_dentry, struct dentry *new_dentry);
struct fid *lookup_fid(struct name *table, ino_t ino, kdev_t *kdev);


#endif /* __NAMECACHE_H */
