/*
 * Copyright (c) 2003-2007 Erez Zadok
 * Copyright (c) 2003-2007 Stony Brook University
 * Copyright (c) 2003-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2007 Gopala Surya
 */

#include "idriver.h"
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>

#ifndef O_LARGEFILE
#define O_LARGEFILE 0100000
#endif

static int fd;

static int fdriver_init(struct parser *p, void *params, struct idriver *next)
{
	char *name = (char *)params;
	if (!name) {
		snprintf(p->err, ERR_SIZE, "ID_FILE: File name not specified");
		return ERR_INVAL;
	}
	if ((fd = open(name, O_RDONLY | O_LARGEFILE)) <= 0) {
		snprintf(p->err, ERR_SIZE, "ID_FILE: Error while opening file %s : %d\n", name, errno);
		return ERR_IO;
	}
	return ERR_OK;

}
static int nread = 0;
static int fdriver_read(struct parser *p, char *buf, int len)
{
	int nbytes = read(fd, buf, len);
	if (nbytes < 0) {
		snprintf(p->err, ERR_SIZE, "ID_FILE: Error while reading %d", errno);
		return -ERR_IO;
	}
	nread += nbytes;
//	fprintf(stderr, "read till now = %d\n", nread);
	return nbytes;
}

static void fdriver_release()
{
	close(fd);
}

struct input_ops fdriver_ops  = {
	init: fdriver_init,
	read: fdriver_read,
	release: fdriver_release
};

struct idriver fdriver = {
	ops: &fdriver_ops
};

struct idriver *file_idriver = &fdriver;
