/*
 * Copyright (c) 2003-2007 Erez Zadok
 * Copyright (c) 2003-2007 Stony Brook University
 * Copyright (c) 2003-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2007 Gopala Surya
 */

#ifndef __TRACEREADER_H
#define __TRACEREADER_H

#include "messages.h"
#include <openssl/evp.h>
#include "stream_header.h"
/*
 * tracereader.h
 * This file defines the external interface of the trace parsing library
 * This parsing library is not intended to be used in a multi-threaded application
 */

#define NONE                0
#define OP_CKSUM_MD5        1
#define OP_UNCOMPRESS       2
#define OP_FILE             3
#define OP_DECRYPT          4
#define MAX_OPS             4

struct decrypt_params {
	char key[EVP_MAX_KEY_LENGTH];
	int keylen;
};

#define HMAC_MAX_KEY_LENGTH 64
struct cksum_params {
	char key[HMAC_MAX_KEY_LENGTH];
	int keylen;
	int blk_size;
};

struct parse_params {

    int ops[MAX_OPS]; /* pre-ops to perform cksum, decompress etc. */
    int n_ops;       /* number of ops actually specified */

    /* file name parameters */
    char *file_name;

    /* checksum params */
    int cksum_type;
    /* compression params */
    int compress_type;
    /* encryption params */
    struct decrypt_params dparams;
    /* cksum params */
    struct cksum_params cparams;
    int bytes_read;
};

int parse(struct parse_params *params);


/*
 * call back functions. The callee should return 0 on success, non zero on fbailure
 */
extern void *on_start_trace_message(op_t type, int pre_or_post,void* context);
extern int on_end_trace_message(op_t type, int pre_or_post, void *context);

// tlv indicates if the data field is of type <type value> or of type <type
// length value>
extern int on_data_field(op_t op, int pre_or_post, int tlv, field_t type, int length, void *value, void *context);
/*
 * error messages
 */
#define ERR_OK 0
#define ERR_IO 1
#define ERR_INCOMPLETE 2
#define ERR_INVALID_LENGTH 3
#define ERR_INVAL 4
#define ERR_INVALID_TYPE  5
#define ERR_VERSION_MISMATCH 6
#define ERR_NOMEM 6
#define ERR_CKSUM_FAILED 7
#define ERR_BLOCK_MISMATCH 8   // block number mismatch

extern void on_error(int error_type, int msg_num, char *desc);


#define WARN_VERSION  1
extern void on_warning(int warning_type, char *desc);

extern int on_header(struct stream_header *header);

#endif
